@@ -15,13 +15,7 @@
 // For the full copyright and license information, please view the LICENSE file
 // that was distributed with this source code.
 
-use std::{
-    ffi::OsStr,
-    fs::read_dir,
-    io::Write,
-    path::{Path, PathBuf},
-    sync::Arc,
-};
+use std::{ffi::OsStr, fs::read_dir, io::Write, path::Path, sync::Arc};
 
 use rayon::{iter::Either, prelude::*};
 use skim::prelude::*;
@@ -31,7 +25,9 @@ use crate::display::display_exec;
 use crate::interactive::SelectionCandidate;
 use crate::lookup::get_versions;
 use crate::utility::httm_is_dir;
-use crate::{Config, DeletedMode, ExecMode, HttmError, PathData, ZFS_HIDDEN_DIRECTORY};
+use crate::{
+    BasicDirEntryInfo, Config, DeletedMode, ExecMode, HttmError, PathData, ZFS_HIDDEN_DIRECTORY,
+};
 
 pub fn display_recursive_exec(
     config: &Config,
@@ -56,34 +52,36 @@ pub fn enumerate_directory(
     tx_item: &SkimItemSender,
     requested_dir: &Path,
 ) -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
-    let (vec_dirs, vec_files): (Vec<PathBuf>, Vec<PathBuf>) = read_dir(&requested_dir)?
-        .flatten()
-        .par_bridge()
-        // never check the hidden snapshot directory for live files (duh)
-        // didn't think this was possible until I saw a SMB share return
-        // a .zfs dir entry
-        .filter(|dir_entry| dir_entry.file_name().as_os_str() != OsStr::new(ZFS_HIDDEN_DIRECTORY))
-        // checking file_type on dir entries is always preferable
-        // as it is much faster than a metadata call on the path
-        .partition_map(|dir_entry| {
-            let path = dir_entry.path();
-            if httm_is_dir(&dir_entry) {
-                Either::Left(path)
-            } else {
-                Either::Right(path)
-            }
-        });
-
-    // need something to hold our threads that we need to wait to have complete,
-    // also very helpful in the case we don't don't needs threads as it can be empty
+    let (vec_dirs, vec_files): (Vec<BasicDirEntryInfo>, Vec<BasicDirEntryInfo>) =
+        read_dir(&requested_dir)?
+            .flatten()
+            .par_bridge()
+            // never check the hidden snapshot directory for live files (duh)
+            // didn't think this was possible until I saw a SMB share return
+            // a .zfs dir entry
+            .filter(|dir_entry| {
+                dir_entry.file_name().as_os_str() != OsStr::new(ZFS_HIDDEN_DIRECTORY)
+            })
+            // checking file_type on dir entries is always preferable
+            // as it is much faster than a metadata call on the path
+            .partition_map(|dir_entry| {
+                let res = BasicDirEntryInfo {
+                    file_name: dir_entry.file_name(),
+                    path: dir_entry.path(),
+                    file_type: dir_entry.file_type().ok(),
+                };
+                if httm_is_dir(&dir_entry) {
+                    Either::Left(res)
+                } else {
+                    Either::Right(res)
+                }
+            });
 
     let spawn_enumerate_deleted = || {
         let config_clone = config.clone();
         let requested_dir_clone = requested_dir.to_path_buf();
         let tx_item_clone = tx_item.clone();
 
-        // thread spawn fn enumerate_directory - permits recursion into dirs without blocking
-        // or blocking when we choose to block
         let _ = enumerate_deleted(config_clone, &requested_dir_clone, &tx_item_clone);
     };
 
@@ -105,7 +103,7 @@ pub fn enumerate_directory(
         }
         ExecMode::Interactive => {
             // combine dirs and files into a vec and sort to display
-            let combined_vec: Vec<PathBuf> = match config.deleted_mode {
+            let combined_vec: Vec<&BasicDirEntryInfo> = match config.deleted_mode {
                 DeletedMode::Only => {
                     spawn_enumerate_deleted();
                     // spawn_enumerate_deleted will send deleted files back to
@@ -118,25 +116,21 @@ pub fn enumerate_directory(
                     // spawn_enumerate_deleted will send deleted files back to
                     // the main thread for us, so we can skip collecting a
                     // vec_deleted here
-                    [&vec_files, &vec_dirs]
-                        .into_par_iter()
-                        .flatten()
-                        .cloned()
-                        .collect()
+                    [&vec_files, &vec_dirs].into_par_iter().flatten().collect()
+                }
+                DeletedMode::Disabled => {
+                    [&vec_files, &vec_dirs].into_par_iter().flatten().collect()
                 }
-                DeletedMode::Disabled => [&vec_files, &vec_dirs]
-                    .into_par_iter()
-                    .flatten()
-                    .cloned()
-                    .collect(),
             };
 
             // don't want a par_iter here because it will block and wait for all
             // results, instead of printing and recursing into the subsequent dirs
-            combined_vec.iter().for_each(|path| {
+            combined_vec.iter().for_each(|basic_dir_entry_info| {
                 let _ = tx_item.send(Arc::new(SelectionCandidate::new(
                     config.clone(),
-                    path.to_path_buf(),
+                    basic_dir_entry_info.file_name.to_owned(),
+                    basic_dir_entry_info.path.to_owned(),
+                    basic_dir_entry_info.file_type,
                     // know this is non-phantom because obtained through dir entry
                     false,
                 )));
@@ -149,7 +143,7 @@ pub fn enumerate_directory(
         // don't want a par_iter here because it will block and wait for all
         // results, instead of printing and recursing into the subsequent dirs
         vec_dirs.iter().for_each(|requested_dir| {
-            let _ = enumerate_directory(config.clone(), tx_item, requested_dir);
+            let _ = enumerate_directory(config.clone(), tx_item, &requested_dir.path);
         });
     }
 
@@ -165,48 +159,45 @@ fn enumerate_deleted(
 ) -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
     let deleted = get_unique_deleted(&config, requested_dir)?;
 
-    let (vec_dirs, vec_files): (Vec<PathBuf>, Vec<PathBuf>) =
-        deleted.into_par_iter().partition_map(|dir_entry| {
-            if httm_is_dir(&dir_entry) {
-                Either::Left(dir_entry.path())
-            } else {
-                Either::Right(dir_entry.path())
-            }
-        });
+    let (vec_dirs, vec_files): (Vec<BasicDirEntryInfo>, Vec<BasicDirEntryInfo>) = deleted
+        .into_par_iter()
+        .partition(|basic_dir_entry_info| httm_is_dir(basic_dir_entry_info));
 
     // need something to hold our thread handles that we need to wait to complete,
     // also very helpful in the case we don't don't needs threads as it can be empty
     let mut vec_handles = Vec::new();
 
     if config.deleted_mode != DeletedMode::DepthOfOne {
-        let _ = vec_dirs.clone().into_iter().for_each(|deleted_dir| {
-            let config_clone = config.clone();
-            let requested_dir_clone = requested_dir.to_path_buf();
-            let tx_item_clone = tx_item.clone();
+        let _ = &vec_dirs
+            .iter()
+            .map(|basic_dir_entry_info| basic_dir_entry_info.path.to_owned())
+            .for_each(|deleted_dir| {
+                let config_clone = config.clone();
+                let requested_dir_clone = requested_dir.to_path_buf();
+                let tx_item_clone = tx_item.clone();
 
-            // thread spawn fn enumerate_directory - permits recursion into dirs without blocking
-            // or blocking when we choose to block
-            let handle = std::thread::spawn(move || {
-                let _ = behind_deleted_dir(
-                    config_clone,
-                    &tx_item_clone,
-                    &deleted_dir,
-                    &requested_dir_clone,
-                );
+                // thread spawn fn enumerate_directory - permits recursion into dirs without blocking
+                // or blocking when we choose to block
+                let handle = std::thread::spawn(move || {
+                    let _ = behind_deleted_dir(
+                        config_clone,
+                        &tx_item_clone,
+                        &deleted_dir,
+                        &requested_dir_clone,
+                    );
+                });
+
+                vec_handles.push(handle);
             });
-
-            vec_handles.push(handle);
-        });
     }
 
     // these are dummy "live versions" values generated to match deleted files
     // which have been found on snapshots, we return to the user "the path that
     // once was" in their browse panel
-    let pseudo_live_versions: Vec<PathBuf> = [&vec_dirs, &vec_files]
+    let pseudo_live_versions: Vec<BasicDirEntryInfo> = [&vec_dirs, &vec_files]
         .into_par_iter()
         .flatten()
-        .filter_map(|path| path.file_name())
-        .map(|file_name| requested_dir.join(file_name))
+        .cloned()
         .collect();
 
     match config.exec_mode {
@@ -248,23 +239,27 @@ fn behind_deleted_dir(
         let deleted_dir_on_snap = &from_deleted_dir.to_path_buf().join(&dir_name);
         let pseudo_live_dir = &from_requested_dir.to_path_buf().join(&dir_name);
 
-        let (vec_dirs, vec_files): (Vec<PathBuf>, Vec<PathBuf>) = read_dir(&deleted_dir_on_snap)?
-            .flatten()
-            .par_bridge()
-            .partition_map(|dir_entry| {
-                let path_buf = dir_entry.path();
-                if httm_is_dir(&dir_entry) {
-                    Either::Left(path_buf)
-                } else {
-                    Either::Right(path_buf)
-                }
-            });
+        let (vec_dirs, vec_files): (Vec<BasicDirEntryInfo>, Vec<BasicDirEntryInfo>) =
+            read_dir(&deleted_dir_on_snap)?
+                .flatten()
+                .par_bridge()
+                .partition_map(|dir_entry| {
+                    let res = BasicDirEntryInfo {
+                        file_name: dir_entry.file_name(),
+                        path: dir_entry.path(),
+                        file_type: dir_entry.file_type().ok(),
+                    };
+                    if httm_is_dir(&dir_entry) {
+                        Either::Left(res)
+                    } else {
+                        Either::Right(res)
+                    }
+                });
 
-        let pseudo_live_versions: Vec<PathBuf> = [&vec_files, &vec_dirs]
+        let pseudo_live_versions: Vec<BasicDirEntryInfo> = [&vec_files, &vec_dirs]
             .into_par_iter()
             .flatten()
-            .filter_map(|path| path.file_name())
-            .map(|file_name| pseudo_live_dir.join(file_name))
+            .cloned()
             .collect();
 
         match config.exec_mode {
@@ -284,11 +279,11 @@ fn behind_deleted_dir(
             _ => unreachable!(),
         }
 
-        vec_dirs.into_iter().for_each(|dir| {
+        vec_dirs.into_iter().for_each(|basic_dir_entry_info| {
             let _ = recurse_behind_deleted_dir(
                 config.clone(),
                 tx_item,
-                Path::new(dir.file_name().unwrap_or_default()),
+                Path::new(&basic_dir_entry_info.file_name),
                 deleted_dir_on_snap,
                 pseudo_live_dir,
             );
@@ -313,13 +308,15 @@ fn behind_deleted_dir(
 
 fn send_deleted_recursive(
     config: Arc<Config>,
-    pathdata: &[PathBuf],
+    pathdata: &[BasicDirEntryInfo],
     tx_item: &SkimItemSender,
 ) -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
-    pathdata.iter().for_each(|path| {
+    pathdata.iter().for_each(|basic_dir_entry_info| {
         let _ = tx_item.send(Arc::new(SelectionCandidate::new(
             config.clone(),
-            path.to_owned(),
+            basic_dir_entry_info.file_name.to_owned(),
+            basic_dir_entry_info.path.to_owned(),
+            basic_dir_entry_info.file_type,
             // know this is_phantom because we know it is deleted
             true,
         )));
@@ -329,11 +326,11 @@ fn send_deleted_recursive(
 
 fn print_deleted_recursive(
     config: Arc<Config>,
-    path_buf_set: &[PathBuf],
+    path_buf_set: &[BasicDirEntryInfo],
 ) -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
     let pseudo_live_set: Vec<PathData> = path_buf_set
         .iter()
-        .map(|path| PathData::from(path.as_path()))
+        .map(|basic_dir_entry_info| PathData::from(basic_dir_entry_info.path.as_path()))
         .collect();
 
     let snaps_and_live_set = get_versions(&config, &pseudo_live_set)?;
